package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AppDTO;
import com.gic.auth.dto.CollaboratorDTO;
import com.gic.auth.dto.CollaboratorListDTO;
import com.gic.auth.qo.CollaboratorListQO;

import java.util.List;

/**
 * 协作人接口
 * @ClassName: CollaboratorApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/11/1 4:15 PM 
 */
public interface CollaboratorApiService {

    /**
     * 新增协作人
     * @Title: save 
     * @Description:
      * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> save(CollaboratorDTO dto);

    /**
     * 分页查询协作人泪奔
     * @Title: page 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.CollaboratorListDTO>> 
      */
    ServiceResponse<Page<CollaboratorListDTO>> page(CollaboratorListQO params);

    /**
     * 查询已购买的一级应用
     * @Title: listApp 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.AppDTO> 
      */
    ServiceResponse<List<AppDTO>> listApp(Integer enterpriseId);

    /**
     * 查询子应用
     * @Title: listSubApp 
     * @Description:
      * @author guojuxing
     * @param appId 父级应用ID
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AppDTO>> 
      */
    ServiceResponse<List<AppDTO>> listSubApp(String appId, Integer enterpriseId);

    /**
     * 查询详情
     * @Title: getDetail 
     * @Description:
      * @author guojuxing
     * @param collaboratorId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.CollaboratorDTO> 
      */
    ServiceResponse<CollaboratorDTO> getDetail(Integer collaboratorId);

    /**
     * 授权
     * @Title: auth 
     * @Description:
      * @author guojuxing
     * @param subAppIds 子应用ID，英文逗号隔开
     * @param menuIds  操作权限ID，英文逗号隔开
     * @param resourcesIds 资源组ID，英文逗号隔开
     * @param collaboratorId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String>  返回日志操作对象
      */
    ServiceResponse<String> auth(String subAppIds, String menuIds, String resourcesIds, Integer collaboratorId);

    /**
     * 取消
     * @Title: cancel 
     * @Description:
      * @author guojuxing
     * @param collaboratorId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> cancel(Integer collaboratorId);
}
